<?php
/*
Plugin Name: Pushover Notifications for WordPress
Plugin URI: http://www.wp-push.com
Description: Pushover Notifications allows your WordPress blog to send push notifications for events happening on your blog straight to your iOS device with the Pushover app. This plugin is not associated with the Pushover Notifications team or Superblock.
Version: 1.8b1
Author: Chris Klosowski
Author URI: http://www.wp-push.com
License: GPLv2 or later
*/

define( 'CKPN_CORE_TEXT_DOMAIN', 'ckpn' );
define( 'CKPN_PATH', plugin_dir_path( __FILE__ ) );
define( 'CKPN_VERSION', '1.8' );
define( 'CKPN_FILE', plugin_basename( __FILE__ ) );

class CKPushoverNotifications {
	private static $ckpn_instance;

	private function __construct() {
		$options = ckpn_get_options();
		
		add_action( 'init', array( $this, 'ckpn_loaddomain', ) );
		add_action( 'cron_schedules', array( $this, 'add_cron_schedule' ) );
		add_action( 'init', array( $this, 'determine_cron_schedule' ) );

		if ( $options['new_user'] ) {
			add_action( 'user_register', array( $this, 'ckpn_user_registration' ) );
		}
		if ( $options['new_comment'] ) {
			add_action( 'comment_post', array( $this, 'ckpn_new_comment' ) );
		}
		if ( $options['password_reset'] ) {
			add_action( 'lostpassword_post', array( $this, 'ckpn_lost_password_request' ) );
		}
		if ( is_admin() ) {
			require_once( CKPN_PATH . '/includes/admin/admin-pages.php' );
			add_action( 'admin_notices', array( $this, 'ckpn_edd_missing_nag' ) );
			add_action( 'admin_init', array( $this, 'ckpn_register_settings' ), 1000, 0 );
			add_action( 'admin_menu', array( $this, 'ckpn_setup_admin_menu' ), 1000, 0 );
			add_filter( 'plugin_action_links', array( $this, 'plugin_settings_links' ), 10, 2 );
			add_filter( 'user_contactmethods', array( $this, 'ckpn_add_contact_item' ), 10, 1 );
		}
	}

	public static function getInstance() {
		if ( !self::$ckpn_instance ) {
			self::$ckpn_instance = new CKPushoverNotifications();
		}

		return self::$ckpn_instance;
	}

	public function add_cron_schedule( $schedules ) {
		$schedules['twicedaily'] = array(
								'interval'  => 43200,
								'display'	=> __( 'Twice Daily', CKPN_CORE_TEXT_DOMAIN ) );

		return $schedules;
	}

	/*
	 * determine_cron_schedule
	 *
	 * Is used to figure out if our cron is already determined and then adds the hook for sending the daily stats
	 */
	public function determine_cron_schedule() {
		$current_options = ckpn_get_options();
		if ( $current_options['plugin_updates'] ) {
			if ( !wp_next_scheduled( 'ckpn_plugin_update_check' ) ) {
				$next_run = time();
				wp_schedule_event( $next_run, 'twicedaily', 'ckpn_plugin_update_check' );
			}
			add_action( 'ckpn_plugin_update_check', array( $this, 'plugin_update_checks' ) );
		}
	}

	public function ckpn_send_notification( $passed_args ) {
		$logentry = '';
		$invalid = false;
		$options = ckpn_get_options();
		$defaults = array(
			'token' => $options['application_key'],
			'user' => NULL );

		$api_args = wp_parse_args( $passed_args, $defaults );

		// Check that we have the required elements
		if ( !isset( $api_args['title'] ) ) {
			$logentry .= $this->log_entry_format( '****** Missing Title ******' );
			$invalid = true;
		}

		if ( !isset( $api_args['message'] ) ) {
			$logentry .= $this->log_entry_format( '****** Missing Message ******' );
			$invalid = true;
		}

		if ( $api_args['user'] == NULL ) { // Use the default key if none was provided
			$api_args['user'] = $options['api_key'];
		}

		if ( !isset( $api_args['user'] ) ) {
			$logentry .= $this->log_entry_format( '****** Missing User Key ******' );
			$invalid = true;
		}

		if ( !isset( $api_args['token'] ) ) {
			$logentry .= $this->log_entry_format( '****** Missing Application Key ******' );
			$invalid = true;
		}

		if ( !$invalid ) {
			/*
			 * Clean house a bit to meet the guidelines of the api
			 *
			 * https://pushover.net/api#limits
			 */
			// Title + Message Limit: 512 Characters
			if ( strlen( $api_args['message'] ) + strlen( $api_args['title'] ) > 512 ) {
				$total_len = strlen( $api_args['message'] ) + strlen( $api_args['title'] );
				$overage = $total_len - 520;
				$api_args['message'] = substr( $api_args['message'], 0, -$overage );
				$last_space = strrpos( $api_args['message'], ' ' );
				$api_args['message'] = substr( $api_args['message'], 0, $last_space ) . '&hellip;';

				$logentry .= $this->log_entry_format( 'Title & Message too long, truncating message' );
			}
			// Let's also get entities ready
			$api_args['title']  = html_entity_decode( $api_args['title'], ENT_QUOTES, 'UTF-8' );
			$api_args['message'] = html_entity_decode( $api_args['message'], ENT_QUOTES, 'UTF-8' );

			// URL limit: 500 Characters
			if ( isset( $api_args['url'] ) && strlen( $api_args['url'] ) > 500 ) {
				unset ( $api_args['url'], $api_args['url_title'] );
				$logentry .= $this->log_entry_format( 'URL was over 500 characters. Removed URL and URL Title' );
			}

			// URL Title limit: 50 Characters
			if ( isset( $api_args['url_title'] ) && strlen( $api_args['url_title'] ) > 50 ) {
				unset ( $api_args['url_title'] );
				$logentry .= $this->log_entry_format( 'URL Title was over 50 characters. Removed URL Title' );
			}

			// Crate the request
			$body = '';
			foreach ( $api_args as $key => $value ) {
				$body .= ( $body == '' ? '' : '&' );
				$body .= urlencode( $key ) . '=' . urlencode( $value );
			}

			$req_args = array( 'body'=>$body );

			if ( $options['sslverify'] == false )
				$req_args['sslverify'] = false;

			if ( $options['logging'] == false )
				$req_args['blocking'] = false;

			// Where the magic happens
			$response = wp_remote_post( 'https://api.pushover.net/1/messages.json', $req_args );

			// From here on out it's just managing logging (if enabled)
			if ( $options['logging'] == 1 ) {
				// If the API Fails for some reason send it via the email system of Pushover
				// This can only be done with logging enabled as it would require a blocking request
				if ( is_wp_error( $response ) ) { 
					$to = $user_key.'@api.pushover.net';
					if ( wp_mail( $to, $subject, $message ) ) {
						$logentry .= $this->log_entry_format( 'Email Notification - ' . $api_args['user'] . ' - Subject: ' . $api_args['title'] );
					} else {
						$logentry .= $this->log_entry_format( '******Email Notification Failed******' );
					}
					$logentry .= $this->log_entry_format( '******API Notification Failed******' );
				} elseif ( $response['response']['code'] == 500 ) {
					$logentry .= $this->log_entry_format( 'API Response - Internal Server Error 500' );
					$logentry .= $this->log_entry_format( 'Typically occurs with non-entitied characters' );
				} else {
					$logentry .= $this->log_entry_format( 'API Notification - ' . $api_args['user'] . ' - Subject: ' . $api_args['title'] );
				}
			}
		}

		if ( $options['logging'] == 1 ) {
			$current_logs = get_option( 'ckpn_logs' );
			$new_logs = $logentry . $current_logs;

			$logs_array = explode( "\n", $new_logs );
			if ( count( $logs_array ) > 100 ) {
				$logs_array = array_slice( $logs_array, 0, 100 );
				$new_logs = implode( "\n", $logs_array );
			}

			update_option( 'ckpn_logs', $new_logs );
		}
	}

	public function ckpn_user_registration( $user_id ) {
		$title = apply_filters( 'ckpn_newuser_subject', get_bloginfo( 'name' ) . ': ' . __( 'New User', CKPN_CORE_TEXT_DOMAIN ) , $user_id);
		
		$user_data = get_userdata( $user_id );
		$message = apply_filters( 'ckpn_newuser_message', sprintf( __( '%s created an account.', CKPN_CORE_TEXT_DOMAIN ), $user_data->user_login ), $user_id, $user_data->user_login);
		
		if ( $title === false || $message === false )
			return;

		$args = apply_filters( 'ckpn_newuser_args', array( 'title' => $title, 'message' => $message ), $user_id );
		
		$this->ckpn_send_notification( $args );
	}

	public function ckpn_new_comment( $comment_id ) {
		$comment_data = get_comment( $comment_id );

		// This is not the comment we're looking for. Move Along.
		if ( $comment_data->comment_approved == 'spam' )
			return;

		switch ( $comment_data->comment_type ) {
			case 'pingback':
				$comment_type = __( 'pingback', CKPN_CORE_TEXT_DOMAIN );
				break;
			case 'trackback':
				$comment_type = __( 'trackback', CKPN_CORE_TEXT_DOMAIN );
				break;
			case 'comment':
			default:
				$comment_type = __( 'comment', CKPN_CORE_TEXT_DOMAIN );
				if ( $comment_data->comment_approved != 0 ){
					$url = get_comment_link( $comment_data );
					$url_title = 'View Comment';
				}
				break;
		}

		$title = apply_filters( 'ckpn_newcomment_subject', get_bloginfo( 'name' ) . ': ' . ucfirst( $comment_type ) );
		$post_data = get_post( $comment_data->comment_post_ID );
		$message = apply_filters( 'ckpn_newcomment_message', sprintf( __( 'by %1$s on %2$s', CKPN_CORE_TEXT_DOMAIN ), $comment_data->comment_author, $post_data->post_title ) );

		// Notify the Admin User
		$args = array( 'title' => $title, 'message' => $message );

		if ( isset( $url ) && isset( $url_title ) ) {
			$args['url'] = $url;
			$args['url_title'] = $url_title;
		}
		
		$this->ckpn_send_notification( $args );

		// Check if we should notify the author as well
		$options = ckpn_get_options();
		if ( $options['notify_authors'] ) {
			$author_user_key = get_user_meta( $post_data->post_author, 'ckpn_user_key', true );
			if ( $author_user_key != '' && $author_user_key != $options['api_key'] ) { // Only send if the user has a key and it's not the same as the admin key
				// Notify the Author their post has a comment
				$args['user'] = $author_user_key;
				$this->ckpn_send_notification( $args );
			}
		}
	}

	public function ckpn_lost_password_request() {
		if ( !empty( $_POST['user_login'] ) ) {
			if ( strpos( $_POST['user_login'], '@' ) ) {
				$user_data = get_user_by( 'email', trim( $_POST['user_login'] ) );
			} else {
				$login = trim( $_POST['user_login'] );
				$user_data = get_user_by( 'login', $login );
			}
		}

		if ( !empty( $user_data ) ) {
			$user_pushover_key = get_user_meta( $user_data->data->ID, 'ckpn_user_key', true );
			if ( $user_pushover_key != '' ) {
				$title = apply_filters( 'ckpn_password_request_subject', get_bloginfo( 'name' ) . ': ' . __( 'Password Reset Request', CKPN_CORE_TEXT_DOMAIN ) );
				$message = apply_filters( 'ckpn_password_request_message', sprintf( __( 'A password reset request was made for your account. If this was not you pelase verify your account is secure.', CKPN_CORE_TEXT_DOMAIN ), $user_data->data->user_login ) );

				$args = array( 'title' => $title, 'message' => $message, 'user' => $user_pushover_key, 'priority' => 1 );
				$this->ckpn_send_notification( $args );
			}
		}
	}

	public function plugin_update_checks() {
		require_once ( ABSPATH . 'wp-admin/includes/update.php' );
		require_once ( ABSPATH . 'wp-admin/includes/admin.php' );

		// Force an update check
		wp_plugin_update_rows();
		$plugin_updates = get_site_transient( 'update_plugins' );
		$plugin_count 	= count( $plugin_updates->response );

		wp_theme_update_rows();
		$theme_updates  = get_site_transient( 'update_themes' );
		$theme_count	=  count( $theme_updates->response );

		if ( empty( $plugin_count ) && empty( $theme_count ) )
			return false;

		$title 		 = apply_filters( 'ckpn_plugin_update_title', get_bloginfo( 'name' ) . ': ' . __( 'Updates Availalbe', CKPN_CORE_TEXT_DOMAIN ) );

		$message  	 = '';
		$plugin_text = _n( 'Plugin has', 'Plugins have', $plugin_count, CKPN_CORE_TEXT_DOMAIN );
		$message 	.= ( !empty( $plugin_count ) ) ? sprintf( __( '%d %s updates', CKPN_CORE_TEXT_DOMAIN ), $plugin_count, $plugin_text ) . "\n" : '';

		$theme_text  = _n( 'Theme has', 'Themes have', $theme_count, CKPN_CORE_TEXT_DOMAIN );
		$message 	.= ( !empty( $theme_count ) ) ? sprintf( __( '%d %s updates', CKPN_CORE_TEXT_DOMAIN ), $theme_count, $theme_text ) : '';
		
		$priority 	 = '1';
		$url 		 = admin_url( 'update-core.php' );
		$url_title	 = 'Update Now';

		$args = array( 'title' => $title, 'message' => $message, 'priority' => $priority, 'url' => $url, 'url_title' => $url_title );
		$this->ckpn_send_notification( $args );
	}

	public function plugin_settings_links( $links, $file ) {
		if ( $file != CKPN_FILE )
			return $links;

		$settings_link = sprintf( '<a href="%s">%s</a>', admin_url( 'options-general.php?page=pushover-notifications' ), __( 'Settings', CKPN_CORE_TEXT_DOMAIN ) );
		$pushover_link = sprintf( '<a href="http://www.pushover.net" target="_blank">%s</a>', __( 'Visit Pushover', CKPN_CORE_TEXT_DOMAIN ) );

		array_unshift( $links, $settings_link );
		$links[] = $pushover_link;

		return $links;
	}

	public function ckpn_setup_admin_menu() {
		add_options_page( __( 'Pushover Notifications', CKPN_CORE_TEXT_DOMAIN ), __( 'Pushover Notifications', CKPN_CORE_TEXT_DOMAIN ), 'administrator', 'pushover-notifications', array( $this, 'determine_tab' ) );
	}

	public function determine_tab() {
		$settings = ckpn_get_options();
		?>
		<div id="icon-options-general" class="icon32"></div><h2><?php _e( 'Pushover Notifications for WordPress', CKPN_CORE_TEXT_DOMAIN ); ?></h2>
		<?php
		$current = ( !isset( $_GET['tab'] ) ) ? 'general' : $_GET['tab'];
		$default_tabs = array(
				'general' => __( 'Settings', CKPN_CORE_TEXT_DOMAIN ),
				'logs' => __( 'Logs', CKPN_CORE_TEXT_DOMAIN ),
				'sysinfo' => __( 'System Info', CKPN_CORE_TEXT_DOMAIN ),
				'licenses' => __( 'Licenses', CKPN_CORE_TEXT_DOMAIN ),
				'extensions' => __( 'Get Extensions', CKPN_CORE_TEXT_DOMAIN )
			);
		
		$tabs = apply_filters( 'ckpn_settings_tabs', $default_tabs );

		?><h2 class="nav-tab-wrapper"><?php
		foreach( $tabs as $tab => $name ){
			$class = ( $tab == $current ) ? ' nav-tab-active' : '';
			echo "<a class='nav-tab$class' href='?page=pushover-notifications&tab=$tab'>$name</a>";
		}
		?>
		</h2>
		<div class="wrap">
		<?php
		if ( !isset( $_GET['tab'] ) || $_GET['tab'] == 'general' ) {
			ckpn_admin_page();
		} else {
			// Extension Devs - Your function that shows the tab contnet needs to be prefaced with 'ckpn_display_' in order to work here.
			$tab_function = 'ckpn_display_'.$_GET['tab'];
			$tab_function();
		}
		?>
		</div>
		<?php
	}

	public function ckpn_add_contact_item( $contact_methods ) {
		$contact_methods['ckpn_user_key'] = 'Pushover User Key';

		return $contact_methods;
	}

	public function ckpn_register_settings() {
		register_setting( 'ckpn-update-options', 'ckpn_pushover_notifications_settings' );
	}

	/*
 	* ckpn_edd_missing_core_nag
 	*
 	* Warns a user if they don't have the core Pushover Notifications plugin.
 	*
 	*
 	* @return void
 	*/
	public function ckpn_edd_missing_nag() {
		if ( !isset($_REQUEST['page'] ) || $_REQUEST['page'] != 'pushover-notifications' )
			return;

		if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) && 
			! is_plugin_active( 'pushover-notifications-edd-ext/pushover-notifications-edd-ext.php' ) ) {
			printf( '<div class="error"> <p> %s </p> </div>', __( 'Get Pushover Notifications for your Easy Digital Downloads Sales with the <a href="https://easydigitaldownloads.com/extension/pushover-notifications/?ref=371" target="_blank">Pushover Notifications Extension</a>.', CKPN_CORE_TEXT_DOMAIN ) );
		}
		if ( is_plugin_active( 'bbpress/bbpress.php' ) && 
			! is_plugin_active( 'pushover-notifications-bbp-ext/pushover-notifications-bbp-ext.php' ) ) {
			printf( '<div class="error"> <p> %s </p> </div>', __( 'Get Pushover Notifications for bbPress with the <a href="https://wp-push.com/extensions/bbpress-extension/" target="_blank">Pushover Notifications Extension</a>.', CKPN_CORE_TEXT_DOMAIN ) );
		}
	}

	public function ckpn_loaddomain() {
		load_plugin_textdomain(CKPN_CORE_TEXT_DOMAIN, false, '/pushover-notifications/languages/' );
	}

	public function get_options() {
		return ckpn_get_options();
	}

	private function log_entry_format( $message = '' ) {
		if ( $message == '' )
			return $message;

		return date( 'm-d-Y H:i:s' ) . ' -- '. $message . "\n";
	}

}

$ckpn_loaded = CKPushoverNotifications::getInstance();

function ckpn_get_options() {
	static $current_options = NULL;

	if ( $current_options !== NULL )
		return $current_options;

	$defaults = array(
			'application_key' 	=> false,
			'api_key'			=> false,
			'new_user'			=> false,
			'new_comment'		=> false,
			'notify_authors'	=> false,
			'password_reset'	=> false,
			'plugin_updates'	=> false,
			'sslverify'			=> false,
			'logging'			=> false
		);

	$defaults = apply_filters( 'ckpn_options_defaults', $defaults );
	
	$options = wp_parse_args( get_option( 'ckpn_pushover_notifications_settings' ), $defaults );

	return $options;
}
